/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1998-1999 Jeremie Miller.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 * --------------------------------------------------------------------------*/
#include <jsm.h>
#include <lber.h>
#include <ldap.h>

#define TERROR_LDAPAUTH          (terror) {450,"Not Member"}


/* --------------------------------------------- */
/* Adjust these setting for your own LDAP server */
/* ---------------------------------------- */
#define SCOPE LDAP_SCOPE_SUBTREE
#define LDAPCHK_SUCCESS 1 /* TRUE */ 
#define LDAPCHK_FAIL    0 /* FALSE */
#define MAX_UN 64
/* ---------------------------------------- */
char HOSTNAME[16];
int PORT_NUMBER;
char BIND_DN[64];
char BIND_PW[32];
char BASE_DN[64];
/* ---------------------------------------- */

/* 
	Check ldap attribute.
		Connects to membership directory, finds t and checks the value of one attribute.
*/
short getLdapAttr(short *cause, char *filter, char *attrName, char *attrValue)
{
	short ret = LDAPCHK_FAIL;
	LDAP *ld;
   LDAPMessage *result, *e;
   char **vals;
	int rc;
   char *attrs[] = {attrName, NULL};	// attributes to return
	// char *upasswd; 			            // password returned by ldap

	*cause = 0;


   /* Get a handle to an LDAP connection. */
   if ((ld = ldap_init(HOSTNAME, PORT_NUMBER)) == NULL) 
   {  
		/*** ERROR: ldap initialize failed ***/
		*cause = 1;
      return LDAPCHK_FAIL;
   }

   /* Bind anonymously to the LDAP server. */
   if ((rc = ldap_simple_bind_s(ld, BIND_DN, BIND_PW)) != LDAP_SUCCESS) 
   {
		/**** ERROR: ldap bind failed */
		*cause = 2;
      return LDAPCHK_FAIL;
   }
   
   /* Search for the entry. */
   if ((rc = ldap_search_s(ld, BASE_DN, SCOPE, filter, attrs, LDAP_NO_LIMIT, &result))
			 != LDAP_SUCCESS) 
   {  /**** ERROR: Failed to find the LDAP entry*/
		*cause = 3;
	   return LDAPCHK_FAIL;   
   } 

   /* Since we are doing a base search, there should be only one matching entry.  */
   e = ldap_first_entry(ld, result); 
   if (e != NULL) 
   {
		/* Get the attributes  */
         	
		if ((vals = ldap_get_values(ld, e, attrName)) != NULL) 
		{
			// attrValue = vals[0];
			sprintf(attrValue,"%s", vals[0]);
			// fprintf(stderr, "LDAP -- val=%s len=%d\n", attrValue, strlen(attrValue));
			ret = LDAPCHK_SUCCESS;
		} 
		else 
		{	/**** ERROR: No such attribute exist */
			*cause = 4;
			ret = LDAPCHK_FAIL;
	   } 
		ldap_value_free(vals);	
   }
	else
	{	/*** ERROR: User Not Found*/
		*cause = 11;
		ret = LDAPCHK_FAIL;
		
	}

	ldap_msgfree(result);
   ldap_unbind(ld);

   return ret;
}

mreturn mod_auth_digest_ldap(mapi m, void *arg)
{
    spool s;
    char *sid;
    char *digest;
    char *mydigest;
	 char passxdb[MAX_UN];
	 char strUname[MAX_UN];
	 short cause = 0;
    
    log_debug("mod_auth_ldap_digest","checking");

    if(jpacket_subtype(m->packet) == JPACKET__GET)
    { /* type=get means we flag that the server can do digest auth */
        xmlnode_insert_tag(m->packet->iq,"digest");
        return M_PASS;
    }

    if ((digest = xmlnode_get_tag_data(m->packet->iq,"digest")) == NULL)
        return M_PASS;

	 /*fprintf(stderr,"ldap_digest ------------ subtype = %d\n",jpacket_subtype(m->packet)); */
	 
	 /*  Check ldap records */
	 strcpy(strUname, "cn=");
    strcat(strUname, m->user->id->user);

	 fprintf(stderr, "digest::user ------> %s\n", strUname);
	 
	 if (getLdapAttr(&cause, strUname, "userpassword", passxdb) == LDAPCHK_SUCCESS)
	 {
		fprintf(stderr, "digest::LDAP Found cause=%d\n", cause);
		log_debug("mod_auth_ldap_eaut","LDAP OK cause=%d", cause);
	 }
	 else
	 {	
		fprintf(stderr, "digest::LDAP NOT FOUND cause=%d\n", cause);
		log_debug("mod_auth_ldap_eaut","LDAP NOT FOUND cause=%d", cause);
		jutil_error(m->packet->x, TERROR_LDAPAUTH);	
		return M_HANDLED;
	 }


	 // xdb = xdb_get(m->si->xc, m->user->id, NS_AUTH);
    // passxdb = xmlnode_get_data(xdb);
	 
    /* get the digest value from the stream */
	 sid = xmlnode_get_attrib(xmlnode_get_tag(m->packet->iq,"digest"), "sid");

    /* Concat the stream id and password */
    /* SHA it up */
    log_debug("mod_auth_digest", "Got SID: %s", sid);
    s = spool_new(m->packet->p);
    spooler(s,sid,passxdb,s);

    mydigest = shahash(spool_print(s));

    /* don't need the xdb data anymore */
    // xmlnode_free(xdb);

    log_debug("mod_auth_ldap_digest","comparing %s %s",digest,mydigest);

    if(digest == NULL || sid == NULL || mydigest == NULL) return M_PASS;

    if(j_strcasecmp(digest, mydigest) != 0)
	 {   jutil_error(m->packet->x, TERROR_LDAPAUTH);
		  fprintf(stderr, "digest::compare failed!\n");
	 }
    else
        jutil_iqresult(m->packet->x);

    return M_HANDLED;
}

void mod_auth_ldap_digest(jsmi si)
{
	xmlnode myx;
	char *pNode;

	log_debug("mod_auth_digest_ldap","init");

	if ((myx = js_config(si, "ldapchk")) != NULL) 
	{
		fprintf(stderr,"ldap settings:\n%s\n", xmlnode2str(myx));
		/* --------------------------------- */
		
		pNode = xmlnode_get_tag_data(myx,"host");
		strcpy(HOSTNAME, pNode);
		fprintf(stderr,"%s\n", HOSTNAME);   
		
		pNode = xmlnode_get_tag_data(myx,"port");
		PORT_NUMBER = atoi(pNode);
		fprintf(stderr,"%d\n", PORT_NUMBER);
		
		pNode = xmlnode_get_tag_data(myx,"binddn");
		strcpy(BIND_DN, pNode);
		fprintf(stderr,"%s\n", BIND_DN);

		pNode = xmlnode_get_tag_data(myx,"bindpw");
		strcpy(BIND_PW,pNode);
		fprintf(stderr,"%s\n", BIND_PW);

		pNode = xmlnode_get_tag_data(myx,"basedn");
		strcpy(BASE_DN, pNode);
		fprintf(stderr,"%s\n", BASE_DN);
	}


    js_mapi_register(si,e_AUTH, mod_auth_digest_ldap, NULL);
}
