/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1998-1999 Jeremie Miller.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 * --------------------------------------------------------------------------*/

#include "jsm.h"

#include <lber.h>
#include <ldap.h>

#define TERROR_LDAPAUTH          (terror) {450,"Not Member"}


/* --------------------------------------------- */
/* Adjust these setting for your own LDAP server */
/* ---------------------------------------- */
#define SCOPE LDAP_SCOPE_SUBTREE
#define LDAPCHK_SUCCESS 1 /* TRUE */ 
#define LDAPCHK_FAIL    0 /* FALSE */
#define MAX_UN 64
/* ---------------------------------------- */
char HOSTNAME[16];
int PORT_NUMBER;
char BIND_DN[64];
char BIND_PW[32];
char BASE_DN[64];
/* ---------------------------------------- */

/* 
	Check ldap attribute.
		Connects to membership directory, finds t and checks the value of one attribute.
*/
short checkLdapAttr(short *cause, char *filter, char *attrName, char *attrValue)
{
	short ret = LDAPCHK_FAIL;
	LDAP *ld;
   LDAPMessage *result, *e;
   char **vals;
	int res;
	int rc;
   char *attrs[] = {attrName, NULL};	// attributes to return
	char *upasswd; 			            // password returned by ldap

	*cause = 0;

   /* Get a handle to an LDAP connection. */
   if ((ld = ldap_init(HOSTNAME, PORT_NUMBER)) == NULL) 
   {  
		/*** ERROR: ldap initialize failed ***/
		*cause = 1;
      return LDAPCHK_FAIL;
   }

   /* Bind anonymously to the LDAP server. */
   if ((rc = ldap_simple_bind_s(ld, BIND_DN, BIND_PW)) != LDAP_SUCCESS) 
   {
		/**** ERROR: ldap bind failed */
		*cause = 2;
      return LDAPCHK_FAIL;
   }
   
   /* Search for the entry. */
   if ((rc = ldap_search_s(ld, BASE_DN, SCOPE, filter, attrs, LDAP_NO_LIMIT, &result))
			 != LDAP_SUCCESS) 
   {  /**** ERROR: Failed to find the LDAP entry*/
		*cause = 3;
	   return LDAPCHK_FAIL;   
   } 

   /* Since we are doing a base search, there should be only one matching entry.  */
   e = ldap_first_entry(ld, result); 
   if (e != NULL) 
   {
		/* Get the attributes  */
         	
		if ((vals = ldap_get_values(ld, e, attrName)) != NULL) 
		{
			upasswd = vals[0];
			res = strncmp(upasswd, attrValue, strlen(attrValue));
			if (res == 0 && (strlen(upasswd) == strlen(attrValue)))
				ret = LDAPCHK_SUCCESS;
			else
			{
			   /**** ERROR: Invalid Password */ 
				*cause = 10;   
				ret = LDAPCHK_FAIL;
			}
		} 
		else 
		{	/**** ERROR: No such attribute exist */
			*cause = 4;
			ret = LDAPCHK_FAIL;
	   } 
		ldap_value_free(vals);	
   }
	else
	{	/*** ERROR: User Not Found*/
		*cause = 11;
		ret = LDAPCHK_FAIL;
		
	}

	ldap_msgfree(result);
   ldap_unbind(ld);

   return ret;
}

/*
	Login Authentication. 
		Calls the ldap control routine to check username/password on ldap.
*/
mreturn mod_auth_ldap_auth(mapi m, void *arg)
{
   short cause;
	char *in_pw = NULL;
	char *in_un = NULL;
	char strUname[MAX_UN];
	
	log_debug("mod_auth_ldap_ereg","ldap auth control ... ");

	if (m->packet->type != JPACKET_IQ) 
		return M_IGNORE;

	// fprintf(stderr,"ldap_auth ------------ subtype = %d\n",jpacket_subtype(m->packet));

	if(jpacket_subtype(m->packet) == JPACKET__GET)
   { /* type=get means we flag that the server can do digest auth */
      xmlnode_insert_tag(m->packet->iq,"password");
      return M_PASS;
   }

	if (jpacket_subtype(m->packet) != JPACKET__SET)
   	return M_PASS;
		
	/* check the packet */ 
#ifdef Z_DEBUG
	// fprintf(stderr,"my packet looks like: %s\n", xmlnode2str(m->packet->x));
#endif

   // log_debug(ZONE,"what does my packet look like: %s",xmlnode2str(m->packet->x));

	if ((in_un = xmlnode_get_tag_data(m->packet->iq,"username")) == NULL) 
	{
		jutil_error(m->packet->x, TERROR_LDAPAUTH);
		return M_HANDLED;
	}
		
	if ((in_pw = xmlnode_get_tag_data(m->packet->iq,"password")) == NULL) 
	{
		jutil_error(m->packet->x, TERROR_LDAPAUTH);
		return M_HANDLED;		
	}

#ifdef Z_DEBUG
	// fprintf(stderr,"-------------ldap_eaut print 1> Username=%s Password=%s\n", in_un, in_pw);
#endif	

	fprintf(stderr,"auth::user ------> %s \n", in_un);
	/*fprintf(stderr,"auth::passw -----> %s \n", in_pw);*/

	/*  Check ldap records */
	strcpy(strUname, "cn=");
   strcat(strUname, in_un);

	/* j1: get username / password user is sending */
	/* -------------------------------------------------------------*/
	if (checkLdapAttr(&cause, strUname, "userpassword", in_pw) == LDAPCHK_SUCCESS)
	{
		fprintf(stderr, "auth::LDAP Found cause=%d\n", cause);
		log_debug("mod_auth_ldap_eaut","LDAP OK cause=%d", cause);
		jutil_iqresult(m->packet->x);
	}
	else
	{	
		fprintf(stderr, "auth::LDAP NOT FOUND cause=%d\n", cause);
		log_debug("mod_auth_ldap_eaut","LDAP NOT FOUND cause=%d", cause);
		jutil_error(m->packet->x, TERROR_LDAPAUTH);	
	}

	return M_HANDLED;   
}

/*
	Block all Registration.
	   Used to Forbid Registering a new user.
		Normally we use this callback to register a new user. But being an ldap auth.
		I am assuming that we are registering users somewhere else, and there is no
		registration through jabber server.
*/
mreturn mod_auth_ldap_noreg(mapi m, void *arg)
{
   
	if (m->packet->type != JPACKET_IQ) 
		return M_IGNORE;
	
	log_debug("mod_auth_ldap_noreg","ldap NO Registration ... ");
		
	if (jpacket_subtype(m->packet) != JPACKET__SET) return M_PASS;

	jutil_error(m->packet->x, TERROR_FORBIDDEN);
	
	return M_HANDLED;
}

/* Check ldap and Register
		Used to check membership before registering the user on jabber server 
*/
mreturn mod_auth_ldap_reg(mapi m, void *arg)
{
   short cause;
	char *in_pw = NULL;
	char *in_un = NULL;
	char strUname[MAX_UN];
	// -------------------------
	
	if (m->packet->type != JPACKET_IQ) 
		return M_IGNORE;
	
	log_debug("mod_auth_ldap_reg","registration ldap check... ");
		
	if (jpacket_subtype(m->packet) != JPACKET__SET) return M_PASS;

	if ((in_un = xmlnode_get_tag_data(m->packet->iq,"username")) == NULL) 
	{
		jutil_error(m->packet->x, TERROR_LDAPAUTH);
		return M_HANDLED;
	}
	
	if ((in_pw = xmlnode_get_tag_data(m->packet->iq,"password")) == NULL) 
	{
		jutil_error(m->packet->x, TERROR_LDAPAUTH);
		return M_HANDLED;		
	}
	
	fprintf(stderr,"reg::user ------> %s \n", in_un);
	/*fprintf(stderr,"reg::passw -----> %s \n", in_pw);*/
	
	strcpy(strUname, "cn=");
   strcat(strUname, in_un);
		
	/* j1: get username / password user is sending */
	/* -------------------------------------------------------------*/
	if (checkLdapAttr(&cause, strUname, "userpassword", in_pw) == LDAPCHK_SUCCESS)
	{
		fprintf(stderr, "reg::LDAP Found cause=%d\n", cause);
		return M_PASS;
	}
	else
	{	
		fprintf(stderr, "reg::LDAP NOT FOUND cause=%d\n", cause);
		jutil_error(m->packet->x, TERROR_LDAPAUTH);
		return M_HANDLED;
	}	
}

/* the main startup/initialization function */
void mod_auth_ldap_z(jsmi si)
{	
	xmlnode myx;
	char *pNode;

	if ((myx = js_config(si, "ldapchk")) != NULL) 
	{
		fprintf(stderr,"ldap settings:\n%s\n", xmlnode2str(myx));
		/* --------------------------------- */
		
		pNode = xmlnode_get_tag_data(myx,"host");
		strcpy(HOSTNAME, pNode);
		fprintf(stderr,"%s\n", HOSTNAME);   
		
		pNode = xmlnode_get_tag_data(myx,"port");
		PORT_NUMBER = atoi(pNode);
		fprintf(stderr,"%d\n", PORT_NUMBER);
		
		pNode = xmlnode_get_tag_data(myx,"binddn");
		strcpy(BIND_DN, pNode);
		fprintf(stderr,"%s\n", BIND_DN);

		pNode = xmlnode_get_tag_data(myx,"bindpw");
		strcpy(BIND_PW,pNode);
		fprintf(stderr,"%s\n", BIND_PW);

		pNode = xmlnode_get_tag_data(myx,"basedn");
		strcpy(BASE_DN, pNode);
		fprintf(stderr,"%s\n", BASE_DN);
	}


	// use mod_auth_ldap_noreg for blocking registration
	js_mapi_register(si, e_REGISTER, mod_auth_ldap_reg, NULL);
    js_mapi_register(si, e_AUTH, mod_auth_ldap_auth, NULL); 
}

